Package: cotram
Title: Count Transformation Models
Version: 0.4-4
Date: 2023-05-25
Authors@R: c(
	person(given = "Sandra", family = "Siegfried", role = c("aut", "cre"), 
	email = "sandra.siegfried@uzh.ch",
	comment = c(ORCID = "0000-0002-7312-1001")),
	person(given = "Luisa", family = "Barbanti", role = "aut",
	comment = c(ORCID = "0000-0001-5352-5802")),
	person(given = "Torsten", family = "Hothorn", role = "aut",
	email = "Torsten.Hothorn@R-project.org",
	comment = c(ORCID = "0000-0001-8301-0471")))
Description: Count transformation models featuring 
        parameters interpretable as discrete hazard ratios, odds ratios,
        reverse-time discrete hazard ratios, or transformed expectations. 
        An appropriate data transformation for a count outcome and
        regression coefficients are simultaneously estimated by maximising
        the exact discrete log-likelihood using the computational framework
        provided in package 'mlt', technical details are given in
        Siegfried & Hothorn (2020) <DOI:10.1111/2041-210X.13383>. 
        The package also contains an experimental implementation of 
        multivariate count transformation models with an application
        to multi-species distribution models <arXiv:2201.13095>.
Depends: R (>= 3.5.0), tram (>= 0.8-0), mlt (>= 1.2-1)
Imports: variables (>= 1.0-2), basefun (>= 1.0-5), alabama, Matrix,
        methods, survival, utils
Suggests: TH.data, knitr, lattice, colorspace, multcomp, mvtnorm,
        numDeriv
VignetteBuilder: knitr
URL: http://ctm.R-forge.R-project.org
Encoding: UTF-8
License: GPL-2
NeedsCompilation: no
Packaged: 2023-05-25 14:59:08 UTC; siegfried
Author: Sandra Siegfried [aut, cre] (<https://orcid.org/0000-0002-7312-1001>),
  Luisa Barbanti [aut] (<https://orcid.org/0000-0001-5352-5802>),
  Torsten Hothorn [aut] (<https://orcid.org/0000-0001-8301-0471>)
Maintainer: Sandra Siegfried <sandra.siegfried@uzh.ch>
Repository: CRAN
Date/Publication: 2023-05-25 15:30:02 UTC
Built: R 4.2.0; ; 2023-07-11 02:00:35 UTC; unix
