Package: cotram
Title: Count Transformation Models
Version: 0.6-1
Date: 2026-06-08
Authors@R: c(
  person(given = "Sandra", family = "Siegfried", role = c("aut", "cre"), 
         email = "sandra.siegfried@alumni.uzh.ch",
	 comment = c(ORCID = "0000-0002-7312-1001")),
  person(given = "Luisa", family = "Barbanti", role = "aut",
	 comment = c(ORCID = "0000-0001-5352-5802")),
  person(given = "Lukas", family = "Graz", role = "aut",
         comment = c(ORCID = "0009-0003-5147-8370")),
  person(given = "Torsten", family = "Hothorn", role = "aut",
         email = "Torsten.Hothorn@R-project.org",
        comment = c(ORCID = "0000-0001-8301-0471")))
Description: Count transformation models featuring 
        parameters interpretable as discrete hazard ratios, odds ratios,
        reverse-time discrete hazard ratios, or transformed expectations. 
        An appropriate data transformation for a count outcome and
        regression coefficients are simultaneously estimated by maximising
        the exact discrete log-likelihood using the computational framework
        provided in package 'mlt', technical details are given in
        Siegfried & Hothorn (2020) <DOI:10.1111/2041-210X.13383>. 
        The package also features
        joint count transformation models with covariate-dependent correlations
        applied to a species community of three aquatic birds
        <DOI:10.48550/arXiv.2201.13095>.
Depends: R (>= 3.5.0), tram (>= 1.3-0), mlt (>= 1.7-0)
Imports: variables (>= 1.0-2), basefun (>= 1.0-5), survival, utils,
        qrng
Suggests: TH.data, knitr, lattice, colorspace, multcomp, mvtnorm,
        numDeriv, viridis, Hmsc
VignetteBuilder: knitr
URL: http://ctm.R-forge.R-project.org
Encoding: UTF-8
License: GPL-2
NeedsCompilation: no
Packaged: 2026-06-08 13:30:21 UTC; hothorn
Author: Sandra Siegfried [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-7312-1001>),
  Luisa Barbanti [aut] (ORCID: <https://orcid.org/0000-0001-5352-5802>),
  Lukas Graz [aut] (ORCID: <https://orcid.org/0009-0003-5147-8370>),
  Torsten Hothorn [aut] (ORCID: <https://orcid.org/0000-0001-8301-0471>)
Maintainer: Sandra Siegfried <sandra.siegfried@alumni.uzh.ch>
Repository: CRAN
Date/Publication: 2026-06-09 07:30:07 UTC
Built: R 4.6.0; ; 2026-06-09 11:55:06 UTC; unix
