
import("mlt")
import("variables")
import("basefun")
import("tram")
importFrom("stats", "coef", "model.frame", "model.matrix", "na.omit",
           "predict", "quantile", "variable.names", "logLik", "vcov", 
           "confint", "simulate")
importFrom("graphics", "plot", "lines", "polygon")
importFrom("stats", "update", "residuals", "pnorm", "weights")
importFrom("graphics", "axis", "box", "points")
importFrom("grDevices", "rgb")
importFrom("utils", "compareVersion", "packageVersion")
importFrom("qrng", "ghalton")

importFrom("survival", "Surv")

export(cotram, mcotram)

S3method("as.mlt", "cotram")
S3method("mkgrid", "cotram")
S3method("logLik", "cotram")
S3method("plot", "cotram")
S3method("predict", "cotram")
S3method("confband", "cotram")
# S3method("simulate", "mcotram")
